#
# spec file for package libteam
#
# Copyright (c) 2024 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%bcond_without python2
# _tmpfilesdir is not defined in systemd macros up to openSUSE 13.2
%{!?_tmpfilesdir: %global _tmpfilesdir %{_libexecdir}/tmpfiles.d }
Name:           libteam
Version:        1.32
Release:        2.3
Summary:        Utilities for controlling 802.1AX team network device
License:        LGPL-2.1-or-later
Group:          System/Kernel
URL:            http://libteam.org/

#Git-Clone:	https://github.com/jpirko/libteam
Source:         http://libteam.org/files/%name-%version.tar.gz
Patch1:         check_if_psr_ops_were_initialized.patch
Patch2:         start_teamd_from_usr_sbin.patch
Patch3:         ignore_ebusy_for_team_hwaddr_set.patch
Patch4:         0001-allow-send_interface-dbus.patch
Patch5:         harden_teamd@.service.patch
Patch6:         better_handle_failures_to_chown.patch
# PATCH-FIX-UPSTREAM bsc#1224798 teamd: lw: arp_ping: bitmask VID in VLAN BPF filter
Patch7:         0001-teamd-lw-arp_ping-bitmask-VID-in-VLAN-BPF-filter.patch
BuildRequires:  doxygen
BuildRequires:  libcap-devel
BuildRequires:  libtool
BuildRequires:  pkg-config
BuildRequires:  swig
BuildRequires:  pkgconfig(dbus-1)
BuildRequires:  pkgconfig(jansson)
BuildRequires:  pkgconfig(libdaemon)
BuildRequires:  pkgconfig(libnl-3.0) >= 3.2.0
BuildRequires:  pkgconfig(libnl-cli-3.0) >= 3.2.0
BuildRequires:  pkgconfig(libnl-genl-3.0) >= 3.2.0
BuildRequires:  pkgconfig(libnl-route-3.0) >= 3.2.0
%if %{with python2}
BuildRequires:  pkgconfig(python2)
%endif
%if 0%{?suse_version} >= 1220
BuildRequires:  systemd-rpm-macros
%endif
%define teamd_user			teamd
%define teamd_group			daemon
%define teamd_daemon_directory		/run/teamd
%define teamd_dbus_policy_directory	%_sysconfdir/dbus-1/system.d
%define teamd_dbus_policy_name		org.libteam.teamd.conf

%description
A library which is the user-space counterpart for the team network
driver, and provides an API to control them.

Linux kernel 3.3 and above offer a so-called "team" network driver -
a lightweight mechanism for bonding multiple interfaces together.
It is a userspace-driven alternative to the existing bonding driver.

%package -n libteam5
Summary:        Library for controlling 802.1AX team network device
Group:          System/Libraries

%description -n libteam5
A library which is the user-space counterpart for the team network
driver, and provides an API to control them.

Linux kernel 3.3 and above offer a so-called "team" network driver -
a lightweight mechanism for bonding multiple interfaces together.
It is a userspace-driven alternative to the existing bonding driver.

%package -n libteamdctl0
Summary:        Library for controlling the team network device daemon
Group:          System/Libraries

%description -n libteamdctl0
Linux kernel 3.3 and above offer a so-called "team" network driver -
a lightweight mechanism for bonding multiple interfaces together.
It is a userspace-driven alternative to the existing bonding driver.

%package devel
Summary:        Development files for libteam
Group:          Development/Libraries/C and C++
Requires:       libteam5 = %version
Requires:       libteamdctl0 = %version

%description devel
A library which is the user-space counterpart for the team network
driver, and provides an API to control them.

This package contains the development headers for the libteam and
libteamdctl libraries.

%package tools
Summary:        Utilities for controlling team network devices
Group:          System/Daemons

%description tools
This package contains frontends to libteam that allow changing
the (team-specific) properties of team devices.
(The general configuration of network devices can be done
through using iproute.)

Linux kernel 3.3 and above offer a so-called "team" network driver -
a lightweight mechanism for bonding multiple interfaces together.
It is a userspace-driven alternative to the existing bonding driver.

%package -n python-libteam
Summary:        Python bindings for libteam
Group:          Development/Languages/Python

%description -n python-libteam
This package should be installed if you want to develop Python
programs that will manipulate team network devices.

%prep
%autosetup -p1

%build
%configure --includedir="%_includedir/%name" --bindir="%_sbindir" \
	--disable-silent-rules --disable-static \
	--with-run-dir="%teamd_daemon_directory" \
	--with-user="%teamd_user" --with-group=%teamd_group
# Use CFLAGS= to kill -Werror
make %{?_smp_mflags} CFLAGS="%optflags"
%if %{with python2}
pushd binding/python/
python ./setup.py build
popd
%endif

%install
b="%buildroot"
%make_install
%if %{with python2}
pushd binding/python/
python ./setup.py install --root="$b" --prefix="%_prefix"
popd
%endif

# Install /usr/lib/tmpfiles.d/libteam.conf
mkdir -p %{buildroot}%{_tmpfilesdir}
cat > %{buildroot}%{_tmpfilesdir}/libteam.conf <<EOF
# See tmpfiles.d(5) for details
# Type(d=directory) Path Mode UID GID Age(until delete when cleaning)
d %teamd_daemon_directory 0755 %teamd_user %teamd_group -
EOF

rm -f "$b/%_libdir"/*.la
%if 0%{?_unitdir:1}
mkdir -p "$b/%_unitdir"
install -pm0644 teamd/redhat/systemd/*.service "$b/%_unitdir/"
%endif
mkdir -p "$b/%teamd_dbus_policy_directory/"
install -pm0644 teamd/dbus/teamd.conf "$b/%teamd_dbus_policy_directory/%teamd_dbus_policy_name"

%check
make check

%pre tools
getent group daemon >/dev/null || %_sbindir/groupadd -r %teamd_group
getent passwd %teamd_user >/dev/null || \
	%_sbindir/useradd -r -g %teamd_group -s /bin/false \
	-c "Teamd daemon user" -d %_localstatedir/lib/empty %teamd_user
%_sbindir/usermod -g %teamd_group %teamd_user 2>/dev/null
test -L %teamd_daemon_directory  || rm -rf %teamd_daemon_directory && :
%if 0%{?_unitdir:1}
%service_add_pre teamd@.service
%endif

%post tools
# Use %%tmpfiles_create when 13.2 is oldest in support scope
/usr/bin/systemd-tmpfiles --create %{_tmpfilesdir}/libteam.conf || :
# reload dbus to apply new teamd's policy
systemctl reload dbus.service 2>/dev/null || :
%if 0%{?_unitdir:1}
%service_add_post teamd@.service
%endif

%preun tools
%if 0%{?_unitdir:1}
%service_del_preun teamd@.service
%endif

%postun tools
%if 0%{?_unitdir:1}
%service_del_postun teamd@.service
%endif
# reload dbus to forget teamd's policy
if [ $1 -eq 0 ]; then
        systemctl reload dbus.service 2>/dev/null || :
fi

%post   -n libteam5 -p /sbin/ldconfig
%postun -n libteam5 -p /sbin/ldconfig
%post   -n libteamdctl0 -p /sbin/ldconfig
%postun -n libteamdctl0 -p /sbin/ldconfig

%files -n libteam5
%_libdir/libteam.so.5*

%files -n libteamdctl0
%_libdir/libteamdctl.so.0*

%files devel
%_includedir/*
%_libdir/libteam.so
%_libdir/libteamdctl.so
%_libdir/pkgconfig/libteam*.pc

%files tools
%_sbindir/bond2team
%_sbindir/team*
%dir %_sysconfdir/dbus-1
%dir %teamd_dbus_policy_directory
%config %teamd_dbus_policy_directory/%teamd_dbus_policy_name
%_mandir/man1/*
%_mandir/man5/*
%_mandir/man8/*
%if 0%{?_unitdir:1}
%_unitdir
%endif
%{_tmpfilesdir}/libteam.conf

%if %{with python2}
%files -n python-libteam
%python_sitearch/*
%endif

%changelog
* Thu May 23 2024 Otto Hollmann <otto.hollmann@suse.com>
- teamd: lw: arp_ping: bitmask VID in VLAN BPF filter [bsc#1224798]
  * Add 0001-teamd-lw-arp_ping-bitmask-VID-in-VLAN-BPF-filter.patch
* Tue Sep 26 2023 Otto Hollmann <otto.hollmann@suse.com>
- update to 1.32:
  * teamd: Add option to change evaluation logic of multiple
    link-watchers
  * teamd: lacp: don't move the port state from disabled when admin
    state
  * teamd: lacp: set port to disabled state during removal
  * libteam: clear changed bits in case of TEAM_IFINFO_CHANGE
  * teamd: stop iterating callbacks when a loop restart is
    requested
  * teamd: do no remove the ports on shutdown with -N
  * binding/python: ifindex 0 is invalid so do not process it
  * libteamdctl: validate the bus name before using it
  * teamd: lacp: increase "min_ports" upper limit to 1024
  * teamd: increase the waitting time for daemon killing
  * Do regard current state when considering port enablement
- Remove 0001-teamd-Add-option-to-change-evaluation-logic-of-multi.patch
* Mon Jun 26 2023 Otto Hollmann <otto.hollmann@suse.com>
- Add option to change evaluation logic of multiple link-watchers (jsc@PED2209)
  Add 0001-teamd-Add-option-to-change-evaluation-logic-of-multi.patch
* Fri Nov  5 2021 Otto Hollmann <otto.hollmann@suse.com>
- teamd: better handle failures to chown(TEAMD_RUN_DIR) during
  teamd_drop_privileges() (bsc#1185424)
  [+ better_handle_failures_to_chown.patch]
* Fri Oct 15 2021 Johannes Segitz <jsegitz@suse.com>
- Added hardening to systemd service(s) (bsc#1181400). Added patch(es):
  * harden_teamd@.service.patch
* Tue Aug 25 2020 Dirk Mueller <dmueller@suse.com>
- update to 1.31:
  * teamd: fix build error in expansion of macro teamd_log_dbgx
  * teamd/lacp: fix segfault due to NULL pointer dereference
  * teamd: fix possible race in master ifname callback
  * Fix ifinfo_link_with_port race condition with newlink
  * Skip setting the same hwaddr to a lag port if not needed
  * teamd/lacp: silence ignore none LACP frames
* Mon Aug 26 2019 Jan Engelhardt <jengelh@inai.de>
- Update to upstream release 1.29
  * teamd: remove port if adding fails
  * teamd: add a default value 1000 for link_watch.interval
  * teamd: fix a json object memleak in get_port_obj()
- Update to upstream release 1.28
  * teamd: do not process lacpdu before the port ifinfo is set
  * teamd: add port_hwaddr_changed for ab runner
  * teamd: add port_hwaddr_changed for lb runner
  * teamd: add port_hwaddr_changed for lacp runner
    name
  * libteam: don't crash when trying to print unregistered device
    teamd_event_watch_ops
  * teamd: add port_master_ifindex_changed for
    syslog
  * teamd: add an option to force log output to stdout, stderr or
    DEFAULT to CURRENT
  * teamd: lacp: send LACPDU when port state transitions from
* Fri Feb 22 2019 Franck Bui <fbui@suse.com>
- Drop use of $FIRST_ARG in .spec
  The use of $FIRST_ARG was probably required because of the
  %%service_* rpm macros were playing tricks with the shell positional
  parameters. This is bad practice and error prones so let's assume
  that no macros should do that anymore and hence it's safe to assume
  that positional parameters remains unchanged after any rpm macro
  call.
* Fri Jan  5 2018 jengelh@inai.de
- Drop /pkg/ subpart from includedir
* Wed Jan  3 2018 tchvatal@suse.com
- Remove defattr that is not really needed
- Add condition around python bindings, they are really based on
  swig code that would need to be rewritten to support python3
* Wed Nov 22 2017 ndas@suse.de
- allow send_interface in dbus conf file for wicked.
  [+ 0001-allow-send_interface-dbus.patch]
* Wed Nov 22 2017 jengelh@inai.de
- Update to new upstream release 1.27
  * teamd: escape some sensitive characters in ifname with
    double quotation marks
  * libteam: resynchronize ifinfo after lost RTNLGRP_LINK
    notifications
  * teamd: check port link_up when a port is added with
    loadbalance runner
* Wed Nov 22 2017 jengelh@inai.de
- Do not suppress errors from useradd
* Wed Nov 22 2017 ndas@suse.de
- sync with SLES changes(bsc#1055708)
  * Add check_if_psr_ops_were_initialized.patch:
  * * Fix teamd segfault when link_watch initialization callbacks fail
  (e.g. lw_psr_port_added() in case of activebackup runner) due to
  an attempt to change hwaddr on an enslaved, link up port device.
  Note: enslavement triggers initialization callbacks.
  * Add ignore_ebusy_for_team_hwaddr_set.patch:
  * * Make PortAdd and PortRemove dbus methods work for all runners.
  Ignore attempts to change hwaddr of an already enslaved devices.
  (fate#318389,fate#317728,fate#316923)
- Add start_teamd_from_usr_sbin.patch:
  * Modify service file to start teamd instance from /usr/sbin.
* Mon Nov 21 2016 jengelh@inai.de
- Update to new upstream release 1.26
  * dbus: don't do <deny send_interface="..." /> in template
    dbus s. f.
  * teamd: do correct l3/l4 tx hashing with vlans
  * teamd: lacp: use original hwaddr as source address in lacpdus
  * libteam: fix TEAM_OPTION_TYPE_BOOL type for big-endian
    architectures
  * teamd: handle vlan 0 packets
  * misc: fix an out-of-bound write with zero-length
    hardware address
  * teamd: fix the issue that network blocks when issuing
    `systemctl stop teamd`
  * teamd: lacp: Do not unselect port if it changes state
    to "expired"
* Thu Feb 11 2016 jengelh@inai.de
- Update to new upstream release 1.22
  * teamd: Fix member port state change on master team admin UP.
  * teamd: add CAP_NET_RAW capability for LACP packet sockets
  * teamd: lacp: update actor state before sending LACP frames
* Wed Sep  9 2015 jengelh@inai.de
- Update to new upstream release 1.18
  * teamd: lacp: change actor system value on team MAC change
  * Fix sending duplicate LACP frames at the start of establishing
  a logical channel.
  * Fix teamd memory corruption issues seen by missing port unlink
  in ifinfo_destroy()
  * libteam: Add check to disallow creating device names longer than
  15 characters.
* Fri Jul 31 2015 jengelh@inai.de
- Update to new upstream release 1.17
  * teamd: lw: nsna_ping: fix NA RX and ARP RX handling
  * libteam: ifinfo: fix rtnl dellink handling
* Sun Mar 29 2015 jengelh@inai.de
- Update to new upstream release 1.16
  * teamdctl: show port link down count in state output
  * teamd: lw: count how many times has been the port down
* Thu Jan  8 2015 jengelh@inai.de
- Update to new upstream release 1.15
  * Topology-aware failover with TIPC
* Mon Jul 21 2014 jengelh@inai.de
- Update to new upstream release 1.11
  * add support for TIPC link watcher
  * teamd quits when the device is removed
  * teamd: add support for checking multiple ports IFLA_PHYS_PORT_ID
* Thu Aug  8 2013 jengelh@inai.de
- Update to new upstream release 1.5
  * lacp: introduce multiple aggregator selection policies
  * lacp: allow to have multiple functions to find out which port
  is better
  * lacp: introduce agg select policy based on per-port config
  options
  * teamd: introduce simple SR-IOV support
  * lacp: do not set state disable if driver does not report speed
  and duplex
  * teamd: add support for multicast group rejoin sending
  * utils: add bond2team conversion tool
* Thu Feb 28 2013 jengelh@inai.de
- Update to new upstream release 1.0 -
  (no changes summary provided; this is the first tarball release)
* Wed Nov 14 2012 jengelh@inai.de
- Update to git snapshot 0+git368 [a62cd9309607c6d72d46ef68403a730a6367b827] -
  no changes summary provided by upstream
- Install systemd service file for teamd
- Build and install python bindings
* Sat May 12 2012 jengelh@inai.de
- Initial package for build.opensuse.org
